@@ -1,16 +1,19 @@
 from __future__ import annotations
-from ast import Dict
 from collections import UserDict
-from typing import Union, Any
+from typing import Any, Callable
 import json
 import copy
 
 
+
 class PathDict(UserDict):
 	"""
 		PathDict wraps a dict, with extra functions like
 		working with paths.
 	"""
+	# Explicitly define data attribute
+	data = None
+
 
 	# FIX wrong behavior of standard library
 	# <a>.pop(<b>, None) returns key error, if <b> not in <a>.
@@ -20,7 +23,6 @@ class PathDict(UserDict):
 
 
 
-
 	def __init__(self, data: dict | PathDict = {}, deep_copy: bool = False):
 		"""
 			Initialize with a dict or another PathDict.
@@ -33,14 +35,18 @@ class PathDict(UserDict):
 		elif isinstance(data, dict):
 			self.data = data
 		else:
-			raise Exception("PathDict init: data must be a dict")
+			raise TypeError("PathDict init: data argument must be a dict or PathDict")
+
 		# Prevent referencing the default value of the data argument
 		if data is self.__init__.__defaults__[0]:
 			self.data = {}
+
 		# If deep_copy is True, make a deep copy of the data
 		if deep_copy:
 			self.data = copy.deepcopy(self.data)
 
+
+
 	@property
 	def dict(self) -> dict:
 		"""
@@ -48,6 +54,8 @@ class PathDict(UserDict):
 		"""
 		return self.data
 
+
+
 	@property
 	def deepcopy(self) -> PathDict:
 		"""
@@ -56,15 +64,17 @@ class PathDict(UserDict):
 		return PathDict(self, deep_copy=True)
 
 
+
 	def __repr__(self) -> str:
 		"""
 			Returns a pretty indented string representation.
 		"""
-		dump = json.dumps(self.data, indent=2, sort_keys=True)
+		dump = json.dumps(self.data, indent=4, sort_keys=True, default=str)
 		return f"PathDict({dump})"
 
 
-	def get_path(self, path: list) -> Union[PathDict, Any]:
+
+	def get_path(self, path: list) -> PathDict | Any:
 		"""
 			Get the value of the json object at the given path
 			<PathDict>.get_path(["1", "2", "3"]) is like calling
@@ -75,18 +85,21 @@ class PathDict(UserDict):
 			return None
 		if path == []:
 			return self
+
+		# Iterate over the path to safely get the value
 		current = self.data
-		for attr in path:
+		for key in path:
 			if not isinstance(current, dict):
-				raise Exception(f"Your path is not a path of dicts (value at key {attr} is of type {type(current)})")
-			if attr not in current:
+				raise Exception(f"The path {path} is not a stack of nested dicts (value at key {key} has type {type(current)})")
+			if key not in current:
 				return None
-			current = current[attr]
+			current = current[key]
 		if isinstance(current, dict):
 			return PathDict(current)
 		return current
 
 
+
 	def set_path(self, path: list, value=None):
 		"""
 			Set the value of the json object at the given path
@@ -104,46 +117,81 @@ class PathDict(UserDict):
 				self.data = value
 			return
 		current = self.data
-		last_path_attr = path.pop()
-		for attr in path:
+		last_path_key = path.pop()
+		for key in path:
 			if not isinstance(current, dict):
 				raise Exception("Can't set the key of a non-dict")
-			current.setdefault(attr, {})
-			current = current[attr]
+			current.setdefault(key, {})
+			current = current[key]
 		if isinstance(value, PathDict):
-			current[last_path_attr] = value.data
+			current[last_path_key] = value.data
 		else:
-			current[last_path_attr] = value
+			current[last_path_key] = value
 
 
-	def apply_at_path(self, path: list, function: callable):
+
+	def _expand_star_path(self, path: list) -> list:
+		"""
+			Expand the given path containing "*" to a list of paths where all "*" have been expanded by all keys.
+		"""
+		paths = [[]]
+		for key in path:
+			if key != "*":
+				# Extend all paths by the key if it is not "*"
+				paths = [path + [key] for path in paths]
+			else:
+				# Now key is "*", expand all paths by all keys at the respective path
+				paths = [p + [k] for p in paths for k in self.get_path(p).keys()]
+
+		# Return empty list if no paths were found
+		paths = paths if paths != [[]] else []
+		return paths
+
+
+
+	def apply_at_path(self, path: list, function: Callable):
 		"""
 			At a given path, apply the given function
 			to the value at that path.
 		"""
-		value = self.get_path(path)
-		self.set_path(path, value=function(value))
+		for expanded_path in self._expand_star_path(path):
+			applied = function(self.get_path(expanded_path))
+			self.set_path(expanded_path, value=applied)
 
 
-	def __getitem__(self, path) -> Any:
+
+	def __getitem__(self, path) -> Any | PathDict:
 		""" Subscript for <PathDict>.get_path() """
 		# If PathDict["key1"], then path="key1"
 		# PathDict["key1", "key2"], then path=tuple("key1", "key2")
 		# We want path to be a list in any case
-		path = list(path) if isinstance(path, tuple) else [path]
+		path = self._convert_path_to_list(path)
 		return self.get_path(path)
 
 
-	def __setitem__(self, path, value):
+
+	def __setitem__(self, path, value: Callable | Any):
 		""" Subscript for <PathDict>.get_path() and <PathDict>.apply_at_path() """
-		path = list(path) if isinstance(path, tuple) else [path]
+		path = self._convert_path_to_list(path)
 		if callable(value):
 			self.apply_at_path(path, function=value)
 		else:
 			self.set_path(path, value=value)
 
 
-	def filter(self, *path, f: callable = None):
+
+	def _convert_path_to_list(self, path) -> list:
+		""" Check path type and convert to list type. """
+		if isinstance(path, list):
+			return path
+		if isinstance(path, tuple):
+			return list(path)
+		# In case of a simple value, return a list with that value
+		return [path]
+
+
+
+	def filter(self, *path, f: Callable = None):
 		"""
 			Only keep the mapping or list elements at path
 			that satisfy f.
@@ -161,7 +209,8 @@ class PathDict(UserDict):
 			self[path] = [x for x in list(path_val) if f(x)]
 
 
-	def filtered(self, *path, f: callable = None):
+
+	def filtered(self, *path, f: Callable = None):
 		"""
 			Like filter, but does not modify this object,
 			but returns a filtered deepcopy.
@@ -171,7 +220,8 @@ class PathDict(UserDict):
 		return deepcopy
 
 
-	def aggregate(self, *path, init=None, f: callable = None):
+
+	def aggregate(self, *path, init=None, f: Callable = None):
 		"""
 			Aggregate a value starting with init at the given path.
 			f takes 3 arguments: key, values, and agg (initialized with init).
@@ -182,4 +232,19 @@ class PathDict(UserDict):
 		agg = init
 		for k, v in path_val.items():
 			agg = f(k, v, agg)
-		return agg
\ No newline at end of file
+		return agg
+
+	def __contains__(self, path) -> bool:
+		"""Check if the path is contained."""
+		path = self._convert_path_to_list(path)
+		temp = self.dict
+		for k in path:
+			if not isinstance(temp, dict):
+				return False
+
+			if k not in temp:
+				return False
+
+			temp = temp[k]
+
+		return True
